@@ -8,6 +8,9 @@
 #
 # Modified Tue  8 Jul 2014: added type="X" to opiInitialise and opiPresent
 # Modified 20 Jul 2014: added maxStim argument for cdTodB conversion
+# Modified September 2016: Added kinetic
+# Modified October 2016: Completely changed kinetic
+# Modified Feburary 2017: Moved kinetic out to simH.r
 #
 # Copyright 2012 Andrew Turpin
 # This program is part of the OPI (http://perimetry.org/OPI).
@@ -24,9 +27,10 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
+#
 
 simH_RT.opiClose         <- function() { return(NULL) }
-simH_RT.opiQueryDevice   <- function() { return (list(type="SimHensonRT")) }
+simH_RT.opiQueryDevice   <- function() { return (list(type="SimHensonRT", isSim=TRUE)) }
 
 if (!exists(".SimHRTEnv"))
     .SimHRTEnv <- new.env(size=5)
@@ -54,9 +58,14 @@ simH_RT.opiInitialize <- function(type="C", cap=6, A=NA, B=NA, display=NULL, max
         warning(msg)
         return(msg)
     }
+    
+    if (type == "N") {  A <- -0.066 ; B <- 2.81 } 
+    else if (type == "G") { A <- -0.098 ;  B <- 3.62    } 
+    else if (type == "C") { A <- -0.081 ;  B <- 3.27    }
 
     if (cap < 0)
         warning("cap is negative in call to opiInitialize (SimHensonRT)")
+    
     .SimHRTEnv$type <- type
     .SimHRTEnv$cap  <-  cap
     .SimHRTEnv$A    <-  A
@@ -65,7 +74,7 @@ simH_RT.opiInitialize <- function(type="C", cap=6, A=NA, B=NA, display=NULL, max
 
     if (type == "X" && (is.na(A) || is.na(B)))
         warning("opiInitialize (SimHenson): you have chosen type X, but one/both A and B are NA")
-
+      
     if(simDisplay.setupDisplay(display))
         warning("opiInitialize (SimHensonRT): display parameter may not contain 4 numbers.")
 
@@ -116,49 +125,61 @@ simH_RT.opiSetBackground <- function(col, gridCol) {
 ################################################################################
 #
 ################################################################################
-simH_RT.opiPresent <- function(stim, nextStim=NULL, fpr=0.03, fnr=0.01, tt=30) { 
+simH_RT.opiPresent <- function(stim, nextStim=NULL, fpr=0.03, fnr=0.01, tt=30, notSeenToSeen=TRUE) { 
                             UseMethod("simH_RT.opiPresent") }
 setGeneric("simH_RT.opiPresent")
 
 #
 # Helper function that allows different coefficients from Table 1 of Henson 2000.
-# Note prob seeing <0 is always false positive rate (but false neg still poss)
+# Note prob seeing <0 is 1 (but false neg still poss)
 # Response time for false positive is uniform sample from .SimHRTEnv$rtFP
 #
-simH_RT.present <- function(db, cap=6, fpr=0.03, fnr=0.01, tt=30, dist, A, B) {
+# @param tt - true threshold (in dB)
+#                If <0 always seen (unless fn) 
+#                If NA always not seen (unless fp)
+# @param dist - distance from threshold in appropriate units
+#
+simH_RT.present <- function(db, fpr=0.03, fnr=0.01, tt=30, dist) {
 
     falsePosRt <- function() {
         if(length(.SimHRTEnv$rtFP) < 2) 
             return(.SimHRTEnv$rtFP)
         else
-            return(sample(.SimHRTEnv$rtFP,1))
+            return(sample(.SimHRTEnv$rtFP,size=1))
     }
 
-    if (tt < 0)         # force false pos if t < 0
+    if (!is.na(tt) && tt < 0)         # force false pos if t < 0
         fpr <- 1.00  
 
+    fn_ret <- fp_ret <- NULL
+
+    if (runif(1) < fpr) 
+        fp_ret <- list(err=NULL, seen=TRUE, time=falsePosRt())  # false P
+
+    if (runif(1) < fnr)
+        fn_ret <- list(err=NULL, seen=FALSE, time=0)            # false N
+
     if (runif(1) < 0.5) {
-            # test fp 
-        if (runif(1) < 2*fpr) {
-            return(list(err=NULL, seen=TRUE, time=falsePosRt()))  # false P
-        }
+        if (!is.null(fp_ret)) return(fp_ret)   # fp first, then fn
+        if (!is.null(fn_ret)) return(fn_ret)
     } else {
-            # test fn 
-        if (runif(1) < fnr) {
-            return(list(err=NULL, seen=FALSE, time=0))                         # false N
-        }
+        if (!is.null(fn_ret)) return(fn_ret)   # fn first, then fp
+        if (!is.null(fp_ret)) return(fp_ret)
     }
 
+    if (is.na(tt))
+        return(list(err=NULL, seen=FALSE, time=0))
+
         # if get to here then need to check Gaussian
         # and if seen=TRUE need to get a time from .SimHRTEnv$rtData
         # assume pxVar is sigma for RT is in sigma units
-
-    pxVar <- min(cap, exp(A*tt + B)) # variability of patient, henson formula 
+    pxVar <- min(.SimHRTEnv$cap, exp(.SimHRTEnv$A*tt + .SimHRTEnv$B)) # variability of patient, henson formula 
+#print(paste(db,tt,pxVar, .SimHRTEnv$cap, .SimHRTEnv$A*tt ,.SimHRTEnv$B))
     if ( runif(1) < 1 - pnorm(db, mean=tt, sd=pxVar)) {
 
         o <- head(order(abs(.SimHRTEnv$rtData$Dist - dist)), 100)
 
-        return(list(err=NULL, seen=TRUE, time=sample(.SimHRTEnv$rtData[o,"Rt"], 1)))
+        return(list(err=NULL, seen=TRUE, time=sample(.SimHRTEnv$rtData[o,"Rt"], size=1)))
     } else {
         return(list(err=NULL, seen=FALSE, time=0))
     }
@@ -188,21 +209,7 @@ simH_RT.opiPresent.opiStaticStimulus <- function(stim, nextStim=NULL, fpr=0.03,
 
     simDisplay.present(stim$x, stim$y, stim$color, stim$duration, stim$responseWindow)
 
-    if (.SimHRTEnv$type == "N") {
-        return(simH_RT.present(cdTodb(stim$level, .SimHRTEnv$maxStim), .SimHRTEnv$cap, fpr, fnr, tt, dist, -0.066, 2.81))
-    } else if (.SimHRTEnv$type == "G") {
-        return(simH_RT.present(cdTodb(stim$level, .SimHRTEnv$maxStim), .SimHRTEnv$cap, fpr, fnr, tt, dist, -0.098, 3.62))
-    } else if (.SimHRTEnv$type == "C") {
-        return(simH_RT.present(cdTodb(stim$level, .SimHRTEnv$maxStim), .SimHRTEnv$cap, fpr, fnr, tt, dist, -0.081, 3.27))
-    } else if (.SimHRTEnv$type == "X") {
-        return(simH_RT.present(cdTodb(stim$level, .SimHRTEnv$maxStim), .SimHRTEnv$cap, fpr, fnr, tt, dist, .SimHRTEnv$A, .SimHRTEnv$B))
-    } else {
-        return ( list(
-            err = "Unknown error in opiPresent() for SimHensonRT",
-            seen= NA,
-            time= NA 
-        ))
-    }
+    return(simH_RT.present(cdTodb(stim$level, .SimHRTEnv$maxStim), fpr, fnr, tt, dist))
 }
 
 ########################################## TO DO !
@@ -213,4 +220,3 @@ simH_RT.opiPresent.opiTemporalStimulus <- function(stim, nextStim=NULL, ...) {
 simH_RT.opiPresent.opiKineticStimulus <- function(stim, nextStim=NULL, ...) {
     stop("ERROR: haven't written simH_RT kinetic persenter yet")
 }
-
